#! /usr/bin/python
# -*- coding: utf-8 -*-
"""
	Python3 is required!

	Calibrating g-code generator by
	2013 Sergey Taranenko(setar@roboforum.ru) - original version in C
	2013 Eduard Bespalov (edwbes@gmail.com) - Python version

	All code released under LGPL license

	Usage: python setar.py [-material=abs] [-printer=makibox] flow|width|speed|bridge|temp|suckL|suckV

	-material=section_name
	      - configuration section to use for the test
	        it could be 'pla', 'abs' or any other specified in calibr3d.ini
	        if not specified the first will be used

	-printer=section_name
	      - configuration with printer related settings

	flow   - find flow coefficient (no need if extruding steps is calibrated in firmware)
	width  - print stripes using different extruding width
	speed  - find max extruding speed
	bridge - find extruding speed for bridges
	temp   - print stripes with different temperature
	suckL  - find suck length
	suckV  - find suck velocity
"""

import sys


if sys.version_info[0] < 3:
	sys.exit('Python 3 is required!')

import math, configparser


#global variables
full_list = ['flow', 'width', 'speed', 'bridge', 'temp', 'suckL', 'suckV']
testName = ""
materialName = ""
printerName = ""

filament_diam = 1.75 # filament diameter
temp = 210 #extruding temperature
bed_temp = 0 #heat bed temperature
fan_speed = 128
#--------- printer settings
path_h = 0.25 # layer height
path_w = 0.45 # layer width
path_w2 = path_w / 2
path_k = 1 #0.875 # layer cross section area in compare to rectangle, KisSlicer =1
fill_speed = 30 # extruding speed
move_speed = 100 # move speed
z_speed = move_speed # z move speed
custom_end = False
custom_start = False
#bed
bed_width = 150.0 # bed width
bed_height = 150.0 # bed height
min_x = 0
min_y = 0
min_z = 0

#-------- test print settings
raft_flow = 1.5 # extruding flow coefficient while printing raft
raft_coef = 0.8 # = 80% raft layer height in % to path_h # = 2 Setar value
raft_filling = 0.5 # 1 - 100%, should be no gaps, 0.5 - 50% 
max_x = min_x + bed_width
max_y = min_y + bed_height
raft_max_y = max_y
params = []
paramsText = ''

#--------- global variables
fil_s = k_s = path_s = 0.0
pos_e = pos_x = pos_y = 0.0
pos_z = min_z #to output first z movement
cur_z = -1
feed_rate = 0 #current speed

#--------- output
picture = []
code = []

def getFloat(param):
	return float(param.split('#', 2)[0].strip())

def getInt(param):
	return int(param.split('#', 2)[0].strip())

def getFList(param):
	return list(filter(None, map(lambda x: float(x.strip()) if x.strip() else '', param.split('#', 2)[0].strip().split(','))))

def getIntList(param):
	return list(filter(None, map(lambda x: int(x.strip()) if x.strip() else '', param.split('#', 2)[0].strip().split(','))))

def setTestParams(test, param):
	global  params, paramsText

	paramsText = '(' + param.split('#', 2)[0].strip() + ')'
	if test == 'flow' or test == 'width' or test == 'suckL' or test == 'suckV' or test == 'bridge':
		params = getFList(param)
	elif test == 'speed' or test == 'temp':
		params = getIntList(param)
	# if len(params):
	# 	print('{:.3f} > {:.3f} > {:.3f}'.format(params[0], params[1], params[2]))

def readConfig(args):
	global filament_diam, temp, bed_temp, fan_speed
	global path_h, path_w, path_k, path_w2, fill_speed, move_speed, z_speed
	global bed_width, bed_height, min_x	, min_y, min_z, max_x, max_y
	global testName, printerName, materialName, params
	global raft_coef, raft_flow, raft_filling, raft_max_y

	#-------- test print settings
	raft_flow = 1.5 # extruding flow coefficient while printing raft
	raft_coef = 0.8 # = 80% raft layer height in % to path_h # = 2 Setar value

	#config.sections()
	#options = config.options(section)
	#config.get(section, option)/getint/getfloat

	config = configparser.RawConfigParser() #ConfigParser(interpolation=None())
	config.read('calibr3d.ini')
	sections = config.sections()

	#find test name
	for arg in args:
		if not arg.startswith('-'):
			testName = arg
			break
	lcTestName = testName.lower()
	section = 'tests'

	for key in config[section]:
		key = key.strip()
		if key == lcTestName:
			setTestParams(testName, config[section][key])
			break;

	#find the printer section from arguments
	section = False
	zSpeedUpdated = False
	for arg in args:
		if arg.startswith('-printer='):
			section = arg[9:]
			break
	if section:
		printerName = section
		for key in config[section]:
			key = key.strip()
			if key == "layer_height":
				path_h = getFloat(config[section][key])
			elif key == "extruding_width":
				path_w = getFloat(config[section][key])
			elif key == "layer_coef":
				path_k = getFloat(config[section][key])
			elif key == "bed_width":
				bed_width = getFloat(config[section][key])
			elif key == "bed_height":
				bed_height = getFloat(config[section][key])
			elif key == "min_x":
				min_x = getFloat(config[section][key])
			elif key == "min_y":
				min_y = getFloat(config[section][key])
			elif key == "min_z":
				min_z = getFloat(config[section][key])
			elif key == "raft_flow":
				raft_flow = getFloat(config[section][key])
			elif key == "raft_height_coef":
				raft_coef = getFloat(config[section][key])
			elif key == "raft_filling":
				raft_filling = getFloat(config[section][key])
			elif key == "fill_speed":
				fill_speed = getInt(config[section][key])
			elif key == "move_speed":
				move_speed = getInt(config[section][key])
				if not zSpeedUpdated:
					z_speed = move_speed
			elif key == "z_speed":
				z_speed = getInt(config[section][key])
				zSpeedUpdated = True
			elif key == "start_code":
				custom_start = config[section][key]
			elif key == "end_code":
				custom_end = config[section][key]
			elif key == lcTestName:
				setTestParams(testName, config[section][key])

	#find the material section from arguments
	section = False
	for arg in args:
		if arg.startswith('-material='):
			section = arg[10:]
			break

	#read filament values
	if section:
		materialName = section
		for key in config[section]:
			key = key.strip()
			if key == "filament":
				filament_diam = getFloat(config[section][key]) # filament diameter
			elif key == "ext_temp":
				temp = getInt(config[section][key])
			elif key == "bed_temp":
				bed_temp = getInt(config[section][key])
			elif key == "fan_speed":
				fan_speed = getInt(config[section][key])
			elif key == lcTestName:
				setTestParams(testName, config[section][key])

	max_x = min_x + bed_width
	max_y = min_y + bed_height
	path_w2 = path_w / 2
	raft_max_y = max_y
	return testName

def xfrange(start, stop=None, step=None):
	"""Like range(), but returns list of floats instead

	All numbers are generated on-demand using generators
	"""

	if stop is None:
		stop = float(start)
		start = 0.0

	if step is None:
		step = 1.0

	cur = float(start)

	if step > 0:
		while cur <= stop + 0.000001:
			yield cur
			cur += step
	else:
		while cur >= stop - 0.000001:
			yield cur
			cur += step

# initialization
def init(test):
	global fil_s, k_s, path_s, params

	fil_s = math.pi * filament_diam * filament_diam /4 # filament cross section area
	path_s = path_k * path_w * path_h # extrusion cross section area
	k_s = path_s / fil_s # filament/extrusion coefficient

	if test == 'flow' or test == 'width' or test == 'speed' or test == 'bridge' or test == 'temp':
		if len(params) < 3:
			sys.exit(test + ' test: not enough parameters or zero in parameters')
	elif test == 'bridge':
		if len(params) < 6:
			sys.exit(test + ' test: not enough parameters or zero in parameters')

# filament consumption for required extrusion length
def filament_length (path_l):
	return float(k_s*path_l)

def head():
	print(';3Dprinter calibrator v0.6 edwbes@gmail.com 2014')
	print(';3Dprinter calibrator v0.0.2 setar@roboforum.ru LGPL 2013')
	print('\n;Printer: ' + printerName)
	print(';Material: ' + materialName)
	print(';Test: {0} {1}'.format(testName, paramsText))

	print('\n;Expected:')
	for line in picture: print('; ' + line)

	print('\n;Filament Diameter = {:.3f} mm'.format(filament_diam))
	print(';Layer height = {:.3f} mm'.format(path_h))
	print(';Extruding width = {:.3f} mm'.format(path_w))
	print(';Path filling = {:.3f}'.format(path_k))
	print(';Temp = {:d}C'.format(temp))
	print(';Bed temp = {:d}C'.format(bed_temp))
	print(';Fill speed = {:d} mm/s'.format(fill_speed))
	print(';Move speed = {:d} mm/s'.format(move_speed))
	print(';Z speed = {:d} mm/s'.format(z_speed))
	print(';Max X = {:d} mm'.format(int(bed_width)))
	print(';Max Y = {:d} mm'.format(int(bed_height)))
	print('\n;--- calculated ---')
	print(';Filament S = {:.4f} mm^2'.format(fil_s))
	print(';Path S = {:.4f} mm^2'.format(path_s))
	print(';Path S/Filament S = {:.4f} mm^2'.format(k_s))

def start_g():
	print('\n;--- start gcode ---')
	print('G28 ; home')
	print('G21 ; unit in mm')
	print('G92 E0 ; set extruder to zero')
	print('M82 ; use absolute distances')
	print('M107 ; fan off')
	print('M104 S{:d}; set temp'.format(temp))
	if bed_temp > 10:
		print('M190 S{:d}; wait bed temp'.format(bed_temp))
	print('\n;--- head clean ---')

	print(';go to the start position')
	print('G1 F{:d}'.format(z_speed * 60))
	print('G1 Z{:.2f}'.format(min_z + 15))
	print('G1 F{:d}'.format(move_speed * 60))
	print('G1 X{:.2f} Y{:.2f}'.format(min_x, min_y))
	if custom_start:
		print(';custom start code ---')
		print(custom_start + '\n')

	print('M109 S{:d}; wait for temperature'.format(temp))
	''' extruding a bubble is commented out
	print('G1 X{:.2f} Y{:.2f} Z{:.2f} ;go near bed level '.format(min_x, min_y + 10, min_z + 0.5))
	print('G1 E5 F100; extrude a bubble')
	print('G4 S5 ; wait 5 sec')
	print('G1 Z{:.2f}'.format(min_z + 5))
	print('G4 S3 ; wait 3 sec')
	print('G1 F15000')
	print('G1 X{:f} Y{:f} Z{:.2f}'.format(min_x, min_y + 20, min_z + path_h))
	print('G1 X{:f} Y{:f}'.format(min_x, min_y + 35))
	'''
	print(';--- end start gcode ---\n')

def stop_g():
	print('\n;--- stop gcode ---')
	print('M400    ; wait for buffer to clear')
	print('M104 S0 ; turn off temperature')
	print('M140 S0 ; turn off bed')
	print('G0 Z{:.2f} F{:d}; ready'.format(pos_z + 5, z_speed))
	print('M84 ; disable motors')
	if custom_end:
		print(';custom end code ---')
		print(custom_end)
	print('')

# extrude line from the current position
def fill_line ( x, y, z, s=fill_speed, flow=1.0):
	"""
	  x,y,z - position to move to
	  s - extruding speed s (mm/s)
	  flow - flow adjustment coefficient, default 1.0
	"""
	global pos_e, pos_x, pos_y, pos_z, feed_rate, code, cur_z

	len_ = math.sqrt((pos_x-x)*(pos_x-x)+(pos_y-y)*(pos_y-y))
	dest_e = pos_e + flow * filament_length(len_)
	f = int(s * 60)

	if feed_rate != f:
		feed_rate = f
		strFeedRate = 'F{:d}'.format(feed_rate)
	else:
		strFeedRate = ''

	strZ = 'Z{:.2f}'.format(z) if cur_z != z else ''

	if z_speed == s:
		code.append('G1 X{:.2f} Y{:.2f} {} E{:.2f} {}'.format(x,y,strZ,dest_e, strFeedRate))
	else:
		if strZ != '':#fixme .length > 0
			code.append('G1 {} F{:d}'.format(strZ, z_speed * 60))
		code.append('G1 X{:.2f} Y{:.2f} E{:.2f} {}'.format(x,y,dest_e, strFeedRate))
	pos_e = dest_e
	pos_x = x
	pos_y = y
	pos_z = z
	cur_z = z

# move to
def move_to ( x, y,  z,  s=move_speed):
	"""
	  x,y,z - position to move to
	  s - moving speed s (mm/s)
	"""
	global pos_x, pos_y, pos_z, feed_rate, code, cur_z

	f = int(s * 60)

	if feed_rate != f:
		feed_rate = f
		strFeedRate = 'F{:d}'.format(feed_rate)
	else:
		strFeedRate = ''

	strZ = 'Z{:.2f}'.format(z) if cur_z != z else ''

	if z_speed == s:
		code.append('G1 X{:.2f} Y{:.2f} {} {}'.format(x,y,strZ, strFeedRate))
	else:
		if strZ:
			code.append('G1 {} F{:d}'.format(strZ, z_speed * 60))
		code.append('G1 X{:.2f} Y{:.2f} {}'.format(x,y, strFeedRate))
	pos_x = x
	pos_y = y
	pos_z = z
	cur_z = z

#reset extruder position
def zero_pos_e():
	global pos_e
	if pos_e != 0:
		code.append('G92 E0')
		pos_e = 0

#extruding width test
def path_width_adjust(w_min, w_max, w_step):
	global pos_z, path_w, raft_max_y

	n = 0
	sep = 0 #additional separator to organize printed stripes
	oldW = path_w #save
	zero_pos_e()
	for path_w in xfrange(w_min, w_max, w_step):
		#5mm per stripe + 2mm gap
		if n * 7 + min_y + 10 + sep < max_y:
			code.append(';--- extruding width {:.3f} start ---'.format(path_w))
			collect_g('{:6.2f}'.format(path_w))
			raft_max_y = n*7 + min_y + 10 + sep
			for y in xfrange( n*7 + min_y + 5 + sep, raft_max_y, 2 * path_w):
				#print 2 lines
				move_to  (min_x, y, pos_z)
				fill_line(max_x, y, pos_z)
				move_to  (max_x, y + path_w, pos_z)
				fill_line(min_x, y + path_w, pos_z)

			code.append(';--- extruding width {:.3f} stop ---'.format(path_w))
			zero_pos_e()
		else:
			raft_max_y = max_y
		n = n + 1
		if n % 5 == 0:
			sep += 3
			collect_g()
	path_w = oldW #restore
	pos_z += path_h #advance to the next layer

#extruding speed test
def path_speed_adjust(s_min, s_max, s_step):
	global pos_z, raft_max_y

	n = 0
	sep = 0 #additional separator to organize printed stripes
	zero_pos_e()
	for speed in range(s_min, s_max + 1, s_step):
		#5mm per stripe + 2mm gap
		if n * 7 + min_y + 10 + sep < max_y:
			code.append(';--- extruding velocity {:d} start ---'.format(speed))
			collect_g('{:d}'.format(speed))
			raft_max_y = n*7 + min_y + 10 + sep
			for y in xfrange( n*7 + min_y + 5 + sep, raft_max_y, 2 * path_w):
				#print 2 lines
				move_to  (min_x, y, pos_z)
				fill_line(max_x, y, pos_z, speed)
				move_to  (max_x, y + path_w, pos_z)
				fill_line(min_x, y + path_w, pos_z, speed)
			code.append(';--- extruding velocity {:d} stop ---'.format(speed))
			zero_pos_e()
		else:
			raft_max_y = max_y
		n = n + 1
		if n % 5 == 0:
			sep += 3
			collect_g()
	pos_z += path_h #advance to the next layer

#print one layer/double wall rectangle
def print_rectangle(side1_y, side2_y):
	'''
	print one layer height/double wall rectangle at requested positions
	side1_y - y position of the start point
	side2_y - y position of the end point
	The rectangle will occupy the whole raft width (X)
	'''

	#perimeter 1 (external)
	x1 = min_x
	x2 = max_x
	y1 = side1_y
	y2 = side2_y
	move_to(x1, y1, pos_z)
	fill_line(x1, y2, pos_z)
	fill_line(x2, y2, pos_z)
	fill_line(x2, y1, pos_z)
	fill_line(x1 + path_w, y1, pos_z)

	#perimeter 2 (inner)
	x1 = x1 + path_w
	x2 = x2 - path_w
	y1 = y1 + path_w
	y2 = y2 - path_w
	fill_line(x1, y2, pos_z)
	fill_line(x2, y2, pos_z)
	fill_line(x2, y1, pos_z)
	fill_line(x1 + path_w, y1, pos_z)

#find extruding speed and flow to print bridges
def bridge_speed_adjust(s_min, s_max, s_step, bridge_len=10.0, flow_min=1.0, flow_step=0.2):
	global pos_z, raft_max_y

	#print rectangles
	#each rectangular - one velocity
	yRange = list(xfrange(min_y + 5, max_y - bridge_len - 5, bridge_len + 3.0))
	sRange = list(range(s_min, s_max + 1, s_step))
	cnt = min(len(sRange), len(yRange))
	yRange = yRange[:cnt]
	sRange = sRange[:cnt]
	raft_max_y = yRange[-1] + bridge_len

	zero_pos_e()
	code.append(';--- bridge walls start ---')
	for z in xfrange(0, 3.0, path_h):
		for y in yRange:
			print_rectangle(y, y + bridge_len)
		pos_z += path_h #advance to the next layer
	code.append(';--- bridge walls stop ---')

	bridgeWidth = 10
	bridgeRange = list(xfrange(min_x + 3, max_x - 3, bridgeWidth + 2))
	flowStr = ' '.join(list(map(lambda x: '{:>5.2f}'.format(x), xfrange(flow_min, flow_min + flow_step * (len(bridgeRange) - 1), flow_step))))
	picture.append('       ' + flowStr)
	n = 0
	for speed in sRange:
		y = yRange[n]
		collect_g('{:d}'.format(speed), len(flowStr))
		flow = flow_min

		for bridge_x in bridgeRange: #10mm bridge width, 2mm - gap between bridges
			code.append(';--- bridge speed: {:d}, flow: {:.2f} start ---'.format(speed, flow))

			#for flow in fRange:
			for x in xfrange(bridge_x, bridge_x + bridgeWidth, 2 * path_w):
				#print 2 lines
				move_to  (x, y, pos_z)
				fill_line(x, y + bridge_len, pos_z, speed, flow)
				move_to  (x + path_w, y + bridge_len, pos_z)
				fill_line(x + path_w, y, pos_z, speed, flow)
			code.append(';--- bridge speed: {:d}, flow: {:.2f} stop ---'.format(speed, flow))
			flow += flow_step
		zero_pos_e()
		n = n + 1
	pos_z += path_h #advance to the next layer

#temperature test
def temp_adjust(t_min, t_max, t_step):
	global pos_z, raft_max_y

	n = 0
	sep = 0 #additional separator to organize printed stripes
	zero_pos_e()
	for xtemp in range(t_min, t_max + 1, t_step):
		#5mm per stripe + 2mm gap
		if n * 7 + min_y + 10 + sep < max_y:
			code.append(';--- temperature {:d} start ---'.format(xtemp))
			# code.append('G1 Z{:f}'.format(pos_z + 5))
			code.append('M109 S{:d}; wait for temperature'.format(xtemp))
			# code.append('G1 Z{:f}'.format(pos_z))

			collect_g('{:d}'.format(xtemp))
			raft_max_y = n*7 + min_y + 10 + sep
			for y in xfrange( n*7 + min_y + 5 + sep, raft_max_y, 2 * path_w):
				#print 2 lines
				move_to  (min_x, y, pos_z)
				fill_line(max_x, y, pos_z)
				move_to  (max_x, y + path_w, pos_z)
				fill_line(min_x, y + path_w, pos_z)
			code.append(';--- temperature {:d} stop ---'.format(xtemp))
			zero_pos_e()
		else:
			raft_max_y = max_y
		n = n + 1
		if n % 5 == 0:
			sep += 3
			collect_g()
	#restore original temperature
	# code.append('G1 Z{:f}'.format(pos_z + 5))
	code.append('M109 S{:d}; wait for temperature'.format(temp))
	# code.append('G1 Z{:f}'.format(pos_z))
	pos_z += path_h #advance to the next layer

#pseudo graphics representing printed test
def collect_g(val='', ln = 30):
	if val:
		picture.append(('{:>6} {:\u00BB<' + str(ln) + '}').format(val, ''))
	else:
		picture.append('')

#flow coefficient test
def path_flow_adjust(f_min, f_max, f_step):
	global pos_z, raft_max_y

	n = 0
	sep = 0 #additional separator to organize printed stripes
	zero_pos_e()
	for flow in xfrange(f_min, f_max, f_step):
		#5mm per stripe + 2mm gap
		if n * 7 + min_y + 10 + sep < max_y:
			code.append(';--- flow {:.2f} start ---'.format(flow))
			collect_g('{:6.2f}'.format(flow))
			raft_max_y = n*7 + min_y + 10 + sep
			for y in xfrange( n*7 + min_y + 5 + sep, raft_max_y, 2 * path_w):
				#print 2 lines
				move_to  (min_x, y, pos_z)
				fill_line(max_x, y, pos_z, fill_speed, flow)
				move_to  (max_x, y + path_w, pos_z)
				fill_line(min_x, y + path_w, pos_z, fill_speed, flow)

			code.append(';--- flow {:.2f} stop ---'.format(flow))
			zero_pos_e()
		else:
			raft_max_y = max_y
		n = n + 1
		if n % 5 == 0:
			sep += 3
			collect_g()
	pos_z += path_h #advance to the next layer

#suck filament
def do_suck(suck, s):
	"""
	suck - suck length, mm
	s - suck speed, mm/s
	"""
	global pos_e, feed_rate
	dest_e = pos_e - suck
	f = int(s * 60)
	feed_rate = f
	code.append('G1 E{:.2f} F{:d}'.format(dest_e, f))
	pos_e = dest_e

#prime filament
def do_prime(prime, s):
	"""
	prime - prime length, mm
	s - prime speed, mm/s
	"""
	global pos_e, feed_rate
	dest_e = pos_e + prime
	f = int(s * 60)
	feed_rate = f
	code.append('G1 E{:.2f} F{:d}'.format(dest_e, f))
	pos_e = dest_e

#test to find the best suck length
def suck_adjust(suck_min, suck_max, suck_step):
	"""
	suck_min - minimum suck length, mm
	suck_max - maximum suck length, mm
	suck_speed - prime speed, mm/s
	"""
	global pos_z, raft_max_y

	suck_speed = fill_speed
	n = 0
	sep = 0 #additional separator to organize printed stripes
	test_line = 0.15 * (max_x - min_x) #15% - start, 70% - gap, 15% - finish
	zero_pos_e()
	for suck in xfrange(suck_min, suck_max, suck_step):
		#5mm per stripe + 2mm gap
		if n * 7 + min_y + 10 + sep < max_y:
			code.append(';--- suck length {:.2f} start ---'.format(suck))
			collect_g('{:6.2f}'.format(suck))
			raft_max_y = n*7 + min_y + 10 + sep
			for y in xfrange( n*7 + min_y + 5 + sep, raft_max_y, 2 * path_w):
				# forth
				move_to  (min_x, y, pos_z)
				fill_line(min_x + test_line, y, pos_z)
				do_suck(suck, suck_speed)
				move_to  (max_x - test_line, y, pos_z)
				do_prime(suck,suck_speed)
				fill_line(max_x, y, pos_z)
				# back
				move_to  (max_x, y + path_w, pos_z)
				fill_line(max_x - test_line, y + path_w, pos_z)
				do_suck(suck, suck_speed)
				move_to  (min_x +  test_line, y + path_w, pos_z)
				do_prime(suck,suck_speed)
				fill_line(min_x, y + path_w, pos_z)

			code.append(';--- suck length {:.2f} stop ---'.format(suck))
			zero_pos_e()
		else:
			raft_max_y = max_y
		n = n + 1
		if n % 5 == 0:
			sep += 3
			collect_g()
	pos_z += path_h #advance to the next layer

#test to find suck speed
def suck_speed_adjust(s_min, s_max, s_step, suck):
	"""
	suck - suck length, mm
	s_min - minimum suck speed, mm/s
	s_max - maximum suck speed, mm/s
	s_step - speed change step
	"""
	global pos_z, raft_max_y

	n = 0
	sep = 0 #additional separator to organize printed stripes
	test_line = 0.15 * (max_x - min_x) #15% - start, 70% - gap, 15% - finish
	zero_pos_e()
	for suck_speed in range(s_min, s_max + 1, s_step):
		#5mm per stripe + 2mm gap
		if n * 7 + min_y + 10 + sep < max_y:
			code.append(';--- suck speed {:d} start ---'.format(suck_speed))
			collect_g('{:6d}'.format(suck_speed))
			raft_max_y = n*7 + min_y + 10 + sep
			for y in xfrange( n*7 + min_y + 5 + sep, raft_max_y, 2 * path_w):
				# forth
				move_to  (min_x, y, pos_z)
				fill_line(min_x + test_line, y, pos_z)
				do_suck(suck, suck_speed)
				move_to  (max_x - test_line, y, pos_z)
				do_prime(suck,suck_speed)
				fill_line(max_x, y, pos_z)
				# back
				move_to  (max_x, y + path_w, pos_z)
				fill_line(max_x - test_line, y + path_w, pos_z)
				do_suck(suck, suck_speed)
				move_to  (min_x +  test_line, y + path_w, pos_z)
				do_prime(suck,suck_speed)
				fill_line(min_x, y + path_w, pos_z)

			code.append(';--- suck speed {:d} stop ---'.format(suck_speed))
			zero_pos_e()
		else:
			raft_max_y = max_y
		n = n + 1
		if n % 5 == 0:
			sep += 3
			collect_g()
	pos_z += path_h #advance to the next layer

#turn fan off/on
def fan(speed):
	fmsg = 'on' if speed > 0 else 'off'
	code.append('M106 S{:d}; fan {:s}'.format(speed, fmsg))

#print raft in the center
def raft():
	global pos_z

	code.append(';--- raft ---')
	flow = raft_flow * raft_coef

	#draw rectangle
	move_to(min_x, min_y, pos_z + path_h * raft_coef)
	do_prime(1,1) #anchor
	code.append('G4 S3 ; wait 3 sec')
	yMax = min( raft_max_y + 5, max_y)
	fill_line(min_x, yMax, pos_z, fill_speed, flow)
	fill_line(max_x, yMax, pos_z, fill_speed, flow)
	fill_line(max_x, min_y, pos_z, fill_speed, flow)
	fill_line(min_x + path_w2, min_y, pos_z, fill_speed, flow)

	yMax = yMax - path_w2
	yMin = min_y + path_w2
	step = path_w / raft_filling
	for x in xfrange(min_x + path_w2, max_x - path_w2, step * 2):
		fill_line(x, yMax, pos_z, fill_speed, flow)
		fill_line(x + step, yMax, pos_z, fill_speed, flow)
		fill_line(x + step, yMin, pos_z, fill_speed, flow)
		fill_line(x + 2*step, yMin, pos_z, fill_speed, flow)
	code.append(';--- end raft ---')
	zero_pos_e()
	pos_z += path_h #advance to the next layer

def runTest(test):
	global pos_z, code

	init(test) # initialisation
	
	pos_z = min_z + path_h * raft_coef + path_h #z level after the raft
	# test selector
	if test == 'flow':
		# find flow coefficient (no need if extruding steps is calibrated in firmware)
		path_flow_adjust(params[0], params[1], params[2])

	elif test == 'width':
		# print stripes using different extruding width (min, max, step)
		path_width_adjust(params[0], params[1], params[2])

	elif test == 'speed':
		# find max extruding speed (min, max, step)
		path_speed_adjust(params[0], params[1], params[2])

	elif test == 'bridge':
		# find extruding speed for bridges (start_speed, max_speed, step)
		bridge_speed_adjust(int(params[0]), int(params[1]), int(params[2]), params[3], params[4], params[5])

	elif test == 'temp':
		# print stripes with different temperature
		temp_adjust(params[0], params[1], params[2])

	elif test == 'suckL':
		suck_adjust(params[0], params[1], params[2]) # find suck value (min, max, step)

	elif test == 'suckV':
		# find suck speed (min, max, step, suck value - value found in suckL)
		suck_speed_adjust(int(params[0]), int(params[1]), int(params[2]), params[3])

	testCode = code
	code = []


	head() # print settings and resulting picture
	start_g() # start g-code
	zero_pos_e() # reset extruder

	pos_z = min_z #init z level
	raft() #print raft
	fan(fan_speed) #turn fan on

	code.extend(testCode)
	for line in code: print(line)

	stop_g() # finishing g-code
	fan(0) #turn fan off

if __name__ == '__main__':
	usage = 'Usage: python setar.py -material=abs -printer=makibox flow|width|speed|bridge|temp|suckL|suckV'
	if len(sys.argv) < 2:
		print(usage)
	else:
		test = readConfig(sys.argv[1:])
		if test in full_list:
			runTest(test)
		else:
			print('Wrong test name: ' + test)
			print(usage)

